// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file CheckerPartitioner.h */

#ifndef AMDIS_CHECKER_PARTITIONER_H
#define AMDIS_CHECKER_PARTITIONER_H

#include "AMDiS_fwd.h"
#include "Global.h"
#include "Initfile.h"
#include "parallel/MeshPartitioner.h"

namespace AMDiS {

  using namespace std;

  class CheckerPartitioner : public MeshPartitioner
  {
  public:
    CheckerPartitioner(MPI::Intracomm *comm)
      : MeshPartitioner(comm),
	mpiRank(mpiComm->Get_rank()),
	mpiSize(mpiComm->Get_size())
    {
      string modestr = "";
      Parameters::get("parallel->partitioner->mode", modestr);

      if (modestr == "")
	mode = 0;
      else if (modestr == "x-stripes")
	mode = 1;
      else if (modestr == "y-stripes")
	mode = 2;
      else if (modestr == "z-stripes")
	mode = 3;
      else
	ERROR_EXIT("No partitioner mode \"%s\"!\n", modestr.c_str());
    }

    ~CheckerPartitioner() {}

    void createInitialPartitioning();

    /// \ref MeshPartitioner::partition
    bool partition(map<int, double> &elemWeights, PartitionMode mode = INITIAL)
    {
      return true;
    }

    void createPartitionMap(map<int, int>& pMap)
    {
      pMap = partitionMap;
    }

  protected:
    int mpiRank, mpiSize;

    /// 0: standard mode: each node gets one box
    /// 1: x-stripes: each node gets one x-stripe of boxes
    /// 2: y-stripes: each node gets one y-stripe of boxes
    int mode;
  };
}

#endif
